#
# tree.py - objects for distro trees and things like them
#
# Will Woods <wwoods@redhat.com>
#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#

from snake.constants import *
import ConfigParser
import os, time

class ComposeTarget(object):
    '''A generic base class for trees and other things built in a compose'''
    def __init__(self,**kwargs):
        self.id=""       # a unique ID string that identifies this object
        self.name=""     # a human-readable string to identify this tree
        self.compose=""  # compose ID
        self.time=0.0    # timestamp
        self.family=""   # family (e.g. Fedora, Red Hat Enterprise Linux)
        self.variant=""  # variant/flavor (e.g. Desktop, AS)
        self.version=""  # tree version. FIXME: what about "4.5.1"?
        self.arch=""     # arch
        self.reldir=""   # (optional) location relative to parent compose
        self.uris=[]  # list of URIs for this particular tree
        # convenience - what type am I?
        #self.type=type(self).__name__
        if 'dir' in kwargs:
            self.loaddir(kwargs['dir'])
        elif 'dict' in kwargs:
            self.__dict__ = kwargs['dict']

    # Provide a convenience member alias for timestamp <-> time
    def __get_timestamp__(self):
        return self.time
    def __set_timestamp__(self, value):
        self.time = value
    timestamp = property(__get_timestamp__, __set_timestamp__)

    def loaddir(self,dir):
        treeinfo=os.path.join(dir,'.treeinfo')
        discinfo=os.path.join(dir,'.discinfo')
        if os.path.exists(treeinfo):
            return self.parsetreeinfo(treeinfo)
        elif os.path.exists(discinfo):
            return self.parsediscinfo(discinfo)

    # TODO - def mktreeinfo to return a treeinfo file?

    def parsetreeinfo(self,treeinfo):
        '''Parse a .treeinfo file to fill in object attributes.
           Argument may be either the path to a file, or an file object.'''

        c = ConfigParser.ConfigParser()
        if hasattr(treeinfo,'readline'):
            c.readfp(treeinfo) # read file-like object
        else:
            c.read(treeinfo)   # try to read it as if it was a filename

        if c.has_option('general','timestamp'):
            self.time = float(c.get('general','timestamp'))
        if c.has_option('general','arch'):
            self.arch = c.get('general','arch')
        if c.has_option('general','family'):
            self.family  = c.get('general','family')
        if c.has_option('general','version'):
            self.version = c.get('general','version')
        if c.has_option('general','variant'):
            self.variant = c.get('general','variant')
        if c.has_option('general','name'):
            self.name = c.get('general','name')
        if c.has_option('general','id'):
            self.id = c.get('general','id')
        if c.has_option('general','uris'):
            self.uris = c.get('general','uris').split()
        if c.has_option('general','kernel_args'):
            self.kernel_args = c.get('general','kernel_args')

        self._fill_in_blanks()
        self.config = c
        return c

    def _fill_in_blanks(self):
        '''Fill in missing 'name' and 'id' attributes by using the timestamp
           and _makename methods'''
        if not self.id:
            self.id = str(self.time)
        if not self.name:
            self.name = self._makename()

    def parsediscinfo(self,discinfo):
        '''Parse a .discinfo file to fill in some object attributes.
           (This kind of sucks since it lacks images/stage2 info - use
           parsetreeinfo instead if possible).
           Argument may be a path to a file, or an open file object.'''
        if hasattr(discinfo,'readline'):
            fp = discinfo
        else:
            fp = open(discinfo)
        self.time = float(fp.readline())
        # Figure out family and version
        desc = fp.readline() # e.g. "Fedora Core 6"
        if desc.startswith("Fedora"):
            self.family = "Fedora"
        elif desc.startswith("Red Hat"):
            self.family = " ".join(desc.split()[:-1])
        self.version = desc.split()[-1]
        # Figure out the arch
        self.arch = fp.readline().strip()
        # Fill in blanks (.name/.id)
        self._fill_in_blanks()
        return fp

    def _makename(self):
        '''Generate a name for this obj using attributes other than self.name.
        Often needed when self.name hasn't be specified directly'''

        strlist = list()
        if self.compose and hasattr(self.compose,'name'):
            strlist.append(self.compose.name)
        else:
            strlist.append(self.family)
            strlist.append(self.version)

            if self.variant and self.variant != self.family:
                strlist.append(self.variant)

            timestamp = time.localtime(float(self.time))
            time_str = time.strftime("%Y%m%d",timestamp)
            strlist.append("%s (%s)" % (self.arch, time_str))

        return ' '.join(strlist)

    def __str__(self):
        if self.name:
            return self.name
        else:
            return _makename()

class Tree(ComposeTarget):
    '''A class that represents an installable tree'''
    # This subclass adds the 'images' dict, which contains boot images
    def __init__(self,**kwargs):
        self.images={}
        self.kernel_args = ""
        ComposeTarget.__init__(self,**kwargs)

    def _fill_in_images(self):
        '''Fill in missing 'images' using an appropriate list of defaults'''
        if not self.images:
            # We can't actually check for the existence of these files.
            # You better handle missing file errors yourself, buddy.
            if self.arch in ('i386', 'x86_64'):
                self.images = {self.arch: IMAGES_X86,'xen': IMAGES_XEN}
            elif self.arch == 'ia64':
                self.images = {self.arch: IMAGES_IA64,'xen': IMAGES_XEN}
            elif self.arch == 'ppc':
                self.images = {self.arch: IMAGES_PPC32, 'ppc64': IMAGES_PPC64}
            elif self.arch.startswith("s390"):
                self.images = {self.arch: IMAGES_S390}

    def parsetreeinfo(self,treeinfo):
        c=ComposeTarget.parsetreeinfo(self,treeinfo)
        for s in c.sections():
            if s.startswith('images-'):
                self.images[s.replace('images-','')]=dict(c.items(s))
        # FIXME: if .treeinfo did not list any [images-*] sections ... should we guess?
        #if not self.images:
        #    self._fill_in_images()

    def parsediscinfo(self,discinfo):
        ComposeTarget.parsediscinfo(self,discinfo)
        # Fill in the 'images' dicts with the standard paths. Hope they're
        # right. This is why discinfo kinda sucks.
        if not self.images:
            self._fill_in_images()

class IsoSet(Tree):
    '''A class that represents a set of ISO images'''
    # This class has a list of iso images, and the boot images live therein.
    # FIXME: We'll need an ISO helper to pull info out of the disc images.
    pass

# Source and Debug are two other things we might need to consider here.
# These are just stubs, defined so we don't forget about them.

class Source(ComposeTarget):
    '''A class for a SRPM dir'''
    pass

class Debug(ComposeTarget):
    '''A class for a debuginfo dir'''
    pass
