#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import os, sys
import optparse
import traceback
import rpmUtils.arch
from socket import gethostname
import snack

from snake.translate import _, N_
from string import joinfields
from urlgrabber import progress

import snake.log
import snake.client
import snake.install
import snake.machineinfo
from snake.constants import *

# import avahi
try:
    import snake.zeroconf
except ImportError:
    has_zeroconf = False
    pass
else:
    has_zeroconf = True

# setup log handler
log = snake.log.setup_logger()

def formatException (type, value, tb):
    lst = traceback.format_tb(tb)
    # Reverse the order that tracebacks are printed
    #lst.reverse() # this makes it too confusing for me (jlaska)
    lst.insert(0, "Snake exception report\n")
    lst.insert(1, 'Traceback (most recent call first):\n')
    lst.extend(traceback.format_exception_only(type, value))
    return lst

def handleException(screen, (type, value, tb)):
    # restore original exception handler
    sys.excepthook = sys.__excepthook__

    # get traceback information
    list = formatException (type, value, tb)
    text = joinfields (list, "")

    # save to local storage first
    out = open("/tmp/snakedump.txt", "w")
    #dumpException (out, text, tb, anaconda)
    out.write(text)
    out.close()

    mainWin = MainExceptionWindow(screen, shortTraceback=text)
    rc = mainWin.run()

    # not sure why this doesn't call __del__ properly
    # del mainWin
    screen.finish()

    log.critical(text)
    return rc

class WaitWindow:
    '''taken almost directly from anaconda/text.py'''
    def pop(self):
        self.screen.popWindow()
        self.screen.refresh()

    def __init__(self, screen, title, text, width):
        self.screen = screen
        if (len(text) < width): width = len(text)

        t = snack.TextboxReflowed(width, text)

        g = snack.GridForm(self.screen, title, 1, 1)
        g.add(t, 0, 0)
        g.draw()
        self.screen.refresh()

def waitWindow(screen, title, text, width):
    return WaitWindow(screen, title, text, width)

class ProgressWindow(progress.BaseMeter):
    def __init__(self, screen, width):
        progress.BaseMeter.__init__(self)
        self.screen = screen
        self.width = width

    def pop(self):
        self.screen.popWindow()
        self.screen.refresh()
        del self.scale
        self.scale = None

    def set(self, amount):
        self.scale.set(amount)
        self.screen.refresh()

    def update(self, amount_read, now=None):
        progress.BaseMeter.update(self, amount_read, now)
        self.set(amount_read)

    def start(self, filename=None, url=None, basename=None, size=None, now=None, text=None):

        # snack doesn't like Scaling based on a None size
        if size is None:
            size = 1

        progress.BaseMeter.start(self, filename, url, basename, size, now, text)

        width = 55
        if (len(text) > width):
            width = min(len(text), self.width)

        t = snack.TextboxReflowed(width, text)

        g = snack.GridForm(self.screen, "Progress", 1, 2)
        g.add(t, 0, 0, (0, 0, 0, 1), anchorLeft=1)

        self.scale = snack.Scale(width, size)
        g.add(self.scale, 0, 1)

        g.draw()
        self.screen.refresh()

def progressWindow(screen, width):
    return ProgressWindow(screen, width)

class SnakeWindow:
    def __init__(self, screen, tui, cfg):
        self.screen = screen
        self.tui = tui
        self.cfg = cfg
        size = snack._snack.size()

class MainExceptionWindow:
    def __init__(self, screen, shortTraceback):
        self.screen = screen
        self.text = "%s\n\n" % shortTraceback

    def run(self):
        #rc = snack.ButtonChoiceWindow(self.screen, _("Exception Occurred"),
        #                              exceptionText, [TEXT_EXIT_BUTTON], self.screen.width-20)

        bb = snack.ButtonBar(self.screen, [TEXT_EXIT_BUTTON])
        t1 = snack.TextboxReflowed(self.screen.width-20, exceptionText, maxHeight = self.screen.height - 16)
        t2 = snack.TextboxReflowed(self.screen.width-20, self.text, maxHeight = self.screen.height - 16)

        g = snack.GridFormHelp(self.screen, _("Exception Occurred"), None, 1, 3)
        g.add(t1, 0, 0, padding = (0, 0, 0, 1))
        g.add(t2, 0, 1, padding = (0, 0, 0, 1))
        g.add(bb, 0, 2, growx = 1)
        rc = bb.buttonPressed(g.runOnce(None, None))

        return TEXT_EXIT_CHECK

    def __del_(self):
        if self.screen:
            self.screen.finish()

class FindServerWindow(SnakeWindow):
    def __call__(self):

        # if no server specified, try to find using zeroconf
        if not self.cfg.server and has_zeroconf:
            '''attempt to find a local snake-server using avahi discovery'''
            window = waitWindow(self.screen, "Searching Network", "Searching your local network for a snake server.", self.tui.width)
            try:
                avahi_client = snake.zeroconf.AvahiClient()
                servers = avahi_client.find_servers()
            except Exception, e:
                servers = list()
                pass
            window.pop() # remove popup when found

            # Were any servers found ... prompt for a selection
            if len(servers) > 1:
                items = [(name,(host,port)) for name,host,port in servers]
                (button, (host,port)) = snack.ListboxChoiceWindow(self.screen,
                    "Choose a server", "Which SNAKE server would you like to connect to?",
                    items, width=self.tui.width, height=min(self.tui.height,len(items)),
                    buttons=[TEXT_OK_BUTTON, TEXT_CANCEL_BUTTON], scroll=1)
                if button in [TEXT_OK_CHECK,None]:
                    (self.cfg.server,self.cfg.port) = (host,port)
            # Just one server found ... use it
            elif len(servers) == 1:
                (garbage, self.cfg.server, self.cfg.port) = servers[0]

        # No server found
        if not self.cfg.server:
            '''no server found, return with error'''
            error_message = "Could not find a snake server. Try using the -s option."
            rc = snack.ButtonChoiceWindow(self.screen,"No Server Found", error_message,
                    [TEXT_EXIT_BUTTON], self.tui.width)
            return TEXT_EXIT_CHECK

        # Check that the server is alive
        if not snake.client.check_server(self.cfg.server, self.cfg.port):
            '''the server is not responding'''
            error_message = "The snake server %s:%s is not responding. " \
                            "Please check your server URL and try again." % (self.cfg.server,self.cfg.port)
            r = snack.ButtonChoiceWindow(self.screen,"Unable To Connect", error_message,
                    [TEXT_EXIT_BUTTON], self.tui.width)
            return TEXT_EXIT_CHECK

        # Connect and save connection object
        self.tui.xmlrpc = snake.client.connect(self.cfg.server, self.cfg.port)
        # Save list of supported methods for future calls
        self.tui.supported_methods = self.tui.xmlrpc.system.listMethods()

        # Update screen text with connection info
        self.screen.drawRootText(0, 0, self.tui.title + " [%s:%s]" % (self.cfg.server, self.cfg.port))

        return TEXT_OK_CHECK

class TreeChoiceWindow(SnakeWindow):
    def __init__(self, *kwargs):
        SnakeWindow.__init__(self, *kwargs)
        self.cfg.tree = None

    def __call__(self):

        args = {"arch": self.cfg.arch}
        treedict = snake.client.treedict(self.tui.xmlrpc, **args)

        if not treedict:
            error_message = "No %s trees found on the snake server %s:%s" % (self.cfg.arch, self.cfg.server, self.cfg.port)
            r = snack.ButtonChoiceWindow(self.screen,"No Trees Found", error_message,
                    [TEXT_EXIT_BUTTON], self.tui.width)
            return TEXT_EXIT_CHECK

        items=list()
        for tid in sorted(treedict.keys(), reverse=True):
            label = str(treedict[tid].name)

            # shrink label to fit the screen
            if len(label) >= self.tui.width:
                label = "%s..." % label[:self.tui.width]

            items.append((label, tid))

        # default selected value
        selected = None
        if treedict.has_key(self.cfg.treeid):
            selected = self.cfg.treeid

        (button, id) = snack.ListboxChoiceWindow(self.screen, "Choose a tree", "What tree would you like to install?",
            items, width=self.tui.width, height=min(self.tui.height,len(items)),
            buttons=[TEXT_OK_BUTTON, TEXT_CANCEL_BUTTON], scroll=1, default=selected)

        if button is None:
            button = TEXT_OK_CHECK

        # remember selection
        self.cfg.treeid = id
        self.cfg.tree = treedict[id]

        return button

class URIChoiceWindow(SnakeWindow):
    def __init__(self, *kwargs):
        SnakeWindow.__init__(self, *kwargs)
        self.cfg.uri = getattr(self.cfg, 'uri', None)

    def __call__(self):

        # TODO sort usefully?
        items = list()
        for u in self.cfg.tree.uris:
            label = u

            # shrink label to fit the screen
            if len(label) >= self.tui.width:
                label = "%s..." % label[:self.tui.width]

            items.append((label, u))

        # default selected value
        selected = None
        if self.cfg.uri in self.cfg.tree.uris:
            selected = self.cfg.uri

        (button, self.cfg.uri) = snack.ListboxChoiceWindow(self.screen, "Choose a mirror",
            "Which mirror would you like to install from?",
            items, width=self.tui.width, height=min(self.tui.height,len(items)),
            buttons=[TEXT_OK_BUTTON, TEXT_BACK_BUTTON, TEXT_CANCEL_BUTTON], scroll=1, default=selected)

        if button is None:
            button = TEXT_OK_CHECK

        return button

class KSMethodWindow(SnakeWindow):
    def __init__(self, *kwargs):
        SnakeWindow.__init__(self, *kwargs)

    def __call__(self):
        items = [(_("initrd - rebuilds the boot initrd to include a /ks.cfg file"),"initrd"),
                 (_("hd - places the ks.cfg file in the /boot partition"), "hd")]

        # probe the server for a list of supported methods
        if "snake.getoption" in self.tui.supported_methods:
            for ksmethod in self.tui.xmlrpc.snake.getoption("ksmethods").split():
                items.append((_("%s - hosted by %s" % (ksmethod, self.cfg.server)), ksmethod))

        (button, self.cfg.ksmethod) = snack.ListboxChoiceWindow(self.screen,
            "Kickstart Delivery", "Choose a kickstart delivery method from the list below.",
            items, width=self.tui.width, height=min(self.tui.height,len(items)),
            buttons=[TEXT_OK_BUTTON, TEXT_BACK_BUTTON, TEXT_CANCEL_BUTTON], scroll=1, default=self.cfg.ksmethod)
        if button is None:
            button = TEXT_OK_CHECK

        return button

class KSChoiceWindow(SnakeWindow):
    def __init__(self, *kwargs):
        SnakeWindow.__init__(self, *kwargs)
        self.cfg.ks = getattr(self.cfg, 'ks', None)

    def __call__(self):

        kslist = self.tui.xmlrpc.kickstart.list()
        items = [('Manual install (no kickstart)',None)]

        selected = None
        for name,desc in kslist:
            label = "%s - %s" % (name,desc)

            # default selected value
            if name == self.cfg.ks:
                selected = name

            # shrink label to fit the screen
            if len(label) >= self.tui.width:
                label = "%s..." % label[:self.tui.width]

            items.append((label, name))

        # build widgets
        text = snack.TextboxReflowed(self.tui.width,
                 _("Here, you may choose a manual installation or select from a "
                   "list of available kickstart installations."))

        # NOTE: Subtracting 2 from self.tui.height to account for the checkbox [] below
        lb = snack.Listbox(min(self.tui.height-2, len(items)), scroll=1, returnExit=1)
        for (label, key) in items:
            lb.append(label, key)
        if selected != None:
            lb.setCurrent(selected)

        # Allow user to specific alternative kickstart delivery
        cb = snack.Checkbox(_("Specify an alternative method for kickstart delivery?"))
        if self.tui.hasScreen(KSMethodWindow):
            cb.setValue("*")

        buttons = snack.ButtonBar(self.screen, [TEXT_OK_BUTTON, TEXT_BACK_BUTTON, TEXT_CANCEL_BUTTON])

        # place widgets on a grid
        g = snack.GridFormHelp(self.screen, _("Select an installation method"), None, 1, 4)
        g.add(text, 0, 0, padding = (0, 0, 0, 1))
        g.add(lb, 0, 1, padding = (0, 0, 0, 1))
        g.add(cb, 0, 2, padding = (0, 0, 0, 1))
        g.add(buttons, 0, 3, growx = 1)

        # run the screen and gather the result
        button = buttons.buttonPressed(g.runOnce())
        self.cfg.ks = lb.current()

        if button is None:
            button = TEXT_OK_CHECK

        # if user requested alternative kickstart delivery, add KSMethodWindow
        if cb.selected() and self.cfg.ks:
            count = 0
            for w in self.tui.windows:
                if w.__name__ == KSChoiceWindow.__name__:
                    self.tui.windows.insert(count+1, KSMethodWindow)
                count += 1
        # Remove KSMethodWindow class
        else:
            count = 0
            for w in self.tui.windows:
                if w.__name__ == KSMethodWindow.__name__:
                    self.tui.windows.remove(w)
                    break
                count += 1

        return button

class BootLoaderAppendWindow(SnakeWindow):
    def __init__(self, *kwargs):
        SnakeWindow.__init__(self, *kwargs)

    def __call__(self):

        self.cfg.fingerprints = snake.machineinfo.get_fingerprints()

        # is the server suggesting any boot arguments?
        if "kernel_args.generate" in self.tui.supported_methods:
            argdata = dict()
            argdata['cmdline'] = getattr(self.cfg,"cmdline","")
            argdata['tree'] = self.cfg.tree.id
            argdata['uri'] = self.cfg.uri
            argdata['fingerprints'] = self.cfg.fingerprints

            if self.cfg.ks:
                argdata['ks'] = self.cfg.ks

            self.cfg.kernel_args = self.tui.xmlrpc.kernel_args.generate(argdata)
        else:
            self.cfg.kernel_args = getattr(self.cfg, "cmdline", "")

        # Inspect /proc/cmdline for certain options (e.g. console=)
        if self.cfg.kernel_args.count("console=") == 0:
            '''if no console= options provided, try to guess'''
            for keyval in open("/proc/cmdline", 'r').read().split():
                if keyval.startswith("console="):
                    self.cfg.kernel_args += " %s" % keyval

        # build widgets
        text = snack.TextboxReflowed(self.tui.width,
                 _("A few systems need to pass special options "
                   "to the kernel at boot time to function "
                   "properly. If you need to pass boot options to the "
                   "kernel, enter them now. If you don't need any or "
                   "aren't sure, leave this blank."))

        # if the server supports saving machine specific kernel args ...
        cb = snack.Checkbox(_("Always use these options for this system?"))
        if "machine.update" not in self.tui.supported_methods:
            cb.setFlags(snack.FLAG_DISABLED, snack.FLAGS_SET)

        entry = snack.Entry(self.tui.width, scroll=1, returnExit=1, text=self.cfg.kernel_args.strip())
        buttons = snack.ButtonBar(self.screen, [TEXT_OK_BUTTON, TEXT_BACK_BUTTON, TEXT_CANCEL_BUTTON])

        # place widgets on a grid
        g = snack.GridFormHelp(self.screen, _("Boot Loader Configuration"), None, 1, 4)
        g.add(text, 0, 0, padding = (0, 0, 0, 1))
        g.add(entry, 0, 1, padding = (0, 0, 0, 1))
        g.add(cb, 0, 2, padding = (0, 0, 0, 1))
        g.add(buttons, 0, 3, growx = 1)

        # run the screen and gather the result
        button = buttons.buttonPressed(g.runOnce(None, None))

        if button is None:
            button = TEXT_OK_CHECK

        # store cmdline arguments
        self.cfg.kernel_args = entry.value()

        # asked to remember these values?
        if cb.selected() \
           and "machine.update" in self.tui.supported_methods \
           and "machine.add" in self.tui.supported_methods \
           and "machine.list" in self.tui.supported_methods:

            # Determine if a matching system registration exists?
            machines = self.tui.xmlrpc.machine.list({"fingerprints": self.cfg.fingerprints})
            if len(machines) == 1:
                m = machines.pop()
                self.tui.xmlrpc.machine.update(m.get("id"), {"bootargs": self.cfg.kernel_args})

            # otherwise, quietly register the current system
            elif len(machines) == 0:
                self.tui.xmlrpc.machine.add(gethostname(), \
                    {"bootargs": self.cfg.kernel_args, \
                     "fingerprints": self.cfg.fingerprints, \
                     "arch": rpmUtils.arch.getBaseArch()})

            # FIXME ... should we quietly fail here and let the snake-server admin examine the logs?
            else:
                raise Exception("Unable to save boot arguments - multiple matches for supplied fingerprints %s" % self.cfg.fingerprints)

        return button

class ReviewWindow(SnakeWindow):
    def __call__(self):

        confirm_msg = '''You are about to configure your system to install this tree:
  %s
which will be installed from the following location:
  %s
'''% (str(self.cfg.tree), self.cfg.uri)
        if self.cfg.ks: confirm_msg += "using the following kickstart:\n  %s\n" % self.cfg.ks
        if self.cfg.kernel_args: confirm_msg += "using the following command-line arguments:\n  %s\n" % self.cfg.kernel_args
        confirm_msg += "\nAre you sure you wish to proceed?"

        rc = snack.ButtonChoiceWindow(self.screen, "Confirm selections?", confirm_msg,
                [TEXT_OK_BUTTON, TEXT_BACK_BUTTON, TEXT_CANCEL_BUTTON], self.tui.width)

        # handle when F12 is pressed
        if rc is None:
            rc = TEXT_OK_CHECK

        # Setup bootloader?
        # FIXME: verify selections and present error if needed
        if rc in [TEXT_OK_CHECK,TEXT_F12_CHECK]:

            fpargs = dict()

            # Are any arch overrides needed?
            # FIXME: this sort of monkeying with arch should really happen in the 
            # client libs, rather than each app..
            from snake.machineinfo import isParaVirt
            if isParaVirt():
                '''paravirt guests will need xen boot images'''
                fpargs["arch"] = "xen"
            else:
                fpargs["arch"] = self.cfg.arch
            if fpargs["arch"] == 'ppc' and os.uname()[4] == 'ppc64':
                fpargs["arch"] = 'ppc64'

            # Generate a kickstart file
            ksdata = None
            if self.cfg.ks:
                # set ver to something usable by pykickstart
                ver = 'devel'
                if self.cfg.tree.version.isdigit():
                    ver = "%s %s" % (self.cfg.tree.family, self.cfg.tree.version)

                # build kickstart method string
                method = ""
                if self.cfg.uri.startswith("http") or self.cfg.uri.startswith("ftp"):
                    method = "url --url=%s" % self.cfg.uri
                elif self.cfg.uri.startswith("nfs"):
                    method = "nfs --server=%s --dir=%s" % tuple(self.cfg.uri[4:].split(':',1))
                else:
                    # TODO: harddrive, cdrom support
                    error_message = "The URI '%s' is not recognized as an anaconda installation source" % self.cfg.uri
                    r = snack.ButtonChoiceWindow(self.screen,"Unrecognized URI", error_message,
                            exit_button, def_width)
                    return (r, "Cancelled.")

                # FIXME: need to emit an error message and drop 'ver' if it turns out
                # we can't generate a kickstart for that version (or if 'ver' is
                # incomprehensible)
                fpargs["ksdata"] = self.tui.xmlrpc.kickstart.generate(self.cfg.ks, \
                                    {"commands": [method], \
                                     "version": ver, \
                                     "fingerprints": self.cfg.fingerprints, \
                                     "ksmeta": dict(arch=fpargs.get("arch"))})

                # was a kickstart method of delivery specified?
                if self.cfg.ksmethod:
                    fpargs["ksmethod"] = self.cfg.ksmethod

                    # if a network ksmethod ... ask the server for a valid URL
                    if self.cfg.ksmethod in ["http", "ftp", "nfs"]:

                        # Determine if a matching system registration exists?
                        # FIXME - just re-use machine id if we registered already
                        machines = self.tui.xmlrpc.machine.list({"fingerprints": self.cfg.fingerprints})

                        # If no matching registration found ... silently register
                        if len(machines) == 0:
                            self.tui.xmlrpc.machine.add(gethostname(), \
                                {"bootargs": self.cfg.kernel_args, \
                                 "fingerprints": self.cfg.fingerprints, \
                                 "arch": rpmUtils.arch.getBaseArch()})

                        if "machine.getksurl" in self.tui.supported_methods:
                            self.cfg.kernel_args += " ks=%s" % self.tui.xmlrpc.machine.getksurl({"fingerprints": self.cfg.fingerprints}, self.cfg.ksmethod)
                        else:
                            error_message = _("Kickstart access method '%s' is not supported on this server." % ksmethod)
                            r = snack.ButtonChoiceWindow(self.screen,"Unsupported kickstart access method", error_message,
                                    exit_button, def_width)
                            return (r, "Cancelled.")

            # setup progress callback
            fpargs["progress_callback"] = progressWindow(self.screen, self.tui.width)

            # Provide fetch_and_prep with any cmdline arguments specified
            fpargs["args"] = self.cfg.kernel_args

            # Give fetch_and_prep the Tree object
            fpargs["tree"] = self.cfg.tree

            try:
                snake.install.fetch_and_prep(self.cfg.uri, **fpargs)
                rc = TEXT_OK_CHECK
            except Exception, e:
                # FIXME: need improved return code handling
                error_message = _("An error occurred while setting up the installer.\n%s" % e)
                r = snack.ButtonChoiceWindow(self.screen,"Error", error_message,
                        [TEXT_EXIT_BUTTON], self.tui.width)
                rc = TEXT_EXIT_CHECK

        return rc

class RebootWindow(SnakeWindow):
    def __call__(self):

        confirm_msg = _("Congratulations, your system is now ready to install. "
                      "The installation will start when your system reboots. \n\n"
                      "Would you like to reboot now?")

        rc = snack.ButtonChoiceWindow(self.screen,"Reboot?", confirm_msg,
                [TEXT_OK_BUTTON, TEXT_EXIT_BUTTON], self.tui.width)

        # Reboot is handled after the snack ui is cleaned up
        #if r in [TEXT_YES_CHECK,TEXT_OK_CHECK]:
        #    os.system("/sbin/reboot")

        return rc

class SnakeTui(object):

    def __init__(self, screen, opt):
        self.title = "SNAKE Client on %s" % opt.arch
        self.screen = screen
        self.size = snack._snack.size()

        # Pull default screen dimensions in to leave room for title/help/scroll elements
        self.width = min(self.size[0]-10,100)
        self.height = self.size[1]-15

        # FIXME: probably shouldn't reuse the OptionParser in this way
        self.cfg = opt

        self.xmlrpc = None # store the handle to the xmlrpc instance

        self.windows = [FindServerWindow,
                        TreeChoiceWindow,
                        URIChoiceWindow,
                        KSChoiceWindow,
                        BootLoaderAppendWindow,
                        ReviewWindow,
                        RebootWindow,
                       ]

        # setup an exception handler to display a formatted traceback
        sys.excepthook = lambda type, value, tb, screen=self.screen: handleException(screen, (type, value, tb))

    def run(self):
        # draw title
        self.screen.drawRootText(0, 0, self.title)

        # loop through ui windows
        step = 0
        while step < len(self.windows):
            win = self.windows[step](self.screen, self, self.cfg)
            rc = win()

            if rc == TEXT_BACK_CHECK:
                step = step - 1
            elif rc in [TEXT_OK_CHECK, TEXT_YES_CHECK, TEXT_F12_CHECK]:
                step = step + 1
            elif rc in [TEXT_EXIT_CHECK,TEXT_CANCEL_CHECK]:
                step = step + 1
                break

        # cleanup snack ui
        self.cleanup()

        # Provide a final status msg
        if step >= len(self.windows):
            if rc in [TEXT_OK_CHECK, TEXT_YES_CHECK, TEXT_F12_CHECK]:
                print "Done.  Your system will reboot to start the install."
                os.system("/sbin/reboot")
            else:
                print "Done.  You may now reboot your system to start the install."
        else:
            print "Cancelled."

        return rc

    def cleanup(self):
        if self.screen:
            self.screen.finish()

    def hasScreen(self, obj):
        for w in self.windows:
            if w.__name__ == obj.__name__:
                return True
        return False

    def __del__(self):
        '''cleanup'''
        self.cleanup()

def setup_option_parser():
    parser=optparse.OptionParser()
    parser.add_option("-s", "--server",
                action="store", dest="server", default=os.environ.get("SNAKE_SERVER",""),
                help="Specify the SNAKE server")
    parser.add_option("-p", "--port",
                action="store", dest="port", default=os.environ.get("SNAKE_PORT","2903"),
                help="The SNAKE xmlrpc port")
    parser.add_option("-a", "--arch",
                action="store", dest="arch", default=rpmUtils.arch.getBaseArch(),
                help="Only include trees of specified architecture (default: %default)")
    parser.add_option("-m", "--ksmethod",
                action="store", dest="ksmethod", default="initrd",
                help="Specify an alternative method for kickstart delivery (default: %default)")
    parser.add_option("-k", "--ks",
                action="store", dest="ks", default=None,
                help="Specify a kickstart name")
    parser.add_option("-t", "--tree",
                action="store", dest="treeid", default=None,
                help="Specify a tree id for an installation source")
    parser.add_option("-u", "--uri",
                action="store", dest="uri", default=None,
                help="Specify a URI for an installation source")
    parser.add_option("-c", "--cmdline",
                action="store", dest="cmdline", default="",
                help="Specify command-line arguments to pass to the installer")
    return parser

def main():
    parser = setup_option_parser()
    (opt,args) = parser.parse_args()

    screen = snack.SnackScreen()

    # TODO: make this a full-fledged SNAKE client (add/remove URIs, etc)
    tui = SnakeTui(screen, opt)
    rc = tui.run()
    sys.exit(rc)

if __name__ == '__main__':

    main()

