#
# uri.py - misc. routines for handing URIs to trees etc.
#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#

import sys, os, urllib2, urlparse, tempfile
import snake.util
from snake.tree import Tree

from snake.translate import _
import logging
import stat
import urlgrabber.grabber as grabber
import urlgrabber.progress as progress
import ftplib

def _fetcherForURI(uri, scratchdir="/tmp"):

    if uri.startswith("http://"):
        return HTTPImageFetcher(uri, scratchdir)
    elif uri.startswith("ftp://"):
        return FTPImageFetcher(uri, scratchdir)
    elif uri.startswith("nfs:"):
        return MountedImageFetcher(uri, scratchdir)
    else:
        raise ValueError("No handler defined for '%s'" % uri)
#        if os.path.isdir(uri):
#            return DirectImageFetcher(uri, scratchdir)
#        else:
#            return MountedImageFetcher(uri, scratchdir)

def uri_to_tree(uri, fetcher=None):
    '''Return a Tree object representing the tree at the given URI'''
    cleanup = False

    # do we need to create a fetcher
    if fetcher is None:
        cleanup = True
        fetcher = _fetcherForURI(uri)
        fetcher.prepareLocation(None)

    # Attempt to create a tree out of the given URI
    t = Tree()
    if fetcher.hasFile('.treeinfo'):
        t.parsetreeinfo(fetcher.loadFile('.treeinfo', None))
    elif fetcher.hasFile('.discinfo'):
        t.parsediscinfo(fetcher.loadFile('.discinfo', None))
    # TODO - what about other methods
    #elif fetcher.hasFile('images/stage2.img'):
    else:
        raise ValueError("No .treeinfo or .discinfo file found for %s" % uri)

    # did we create a fetcher, if so clean it up
    if cleanup:
        fetcher.cleanupLocation()

    return t

# This is a generic base class for fetching/extracting files from
# a media source, such as CD ISO, NFS server, or HTTP/FTP server
class ImageFetcher:

    def __init__(self, location, scratchdir=None):
        self.location = location
        self.scratchdir = scratchdir

    def saveTemp(self, fileobj, prefix):
        (fd, fn) = tempfile.mkstemp(prefix="snake-" + prefix, dir=self.scratchdir)
        block_size = 16384
        try:
            while 1:
                buff = fileobj.read(block_size)
                if not buff:
                    break
                os.write(fd, buff)
        finally:
            os.close(fd)
        return fn

    def prepareLocation(self, progresscb):
        return True

    def cleanupLocation(self):
        pass

    def loadFile(self, filename, progresscb):
        return self._acquireFile(filename, progresscb, save=False)

    def saveFile(self, filename, progresscb):
        return self._acquireFile(filename, progresscb, save=True)

    def _acquireFile(self, src, progresscb):
        raise "Must be implemented in subclass"

    def hasFile(self, src):
        raise "Must be implemented in subclass"

# Base class for downloading from FTP / HTTP
class URIImageFetcher(ImageFetcher):

    def prepareLocation(self, progresscb):
        try:
            grabber.urlopen(self.location,
                            progress_obj = progresscb,
                            text = _("Verifying install location..."))
            return True
        except IOError, e:
            logging.debug("Opening URL %s failed." % (self.location,) + " " + str(e))
            raise ValueError(_("Opening URL %s failed: %s") % \
                              (self.location, e))
            return False

    def _acquireFile(self, filename, progresscb, save=False):
        file = None
        try:
            base = os.path.basename(filename)
            logging.debug("Fetching URI " + self.location + "/" + filename)
            try:
                file = grabber.urlopen(self.location + "/" + filename,
                                       progress_obj = progresscb, \
                                       text = _("Retrieving file %s...") % base)
            except IOError, e:
                raise ValueError, _("Couldn't acquire file %s: %s") %\
                                  ((self.location + "/" + filename), str(e))
            if save:
                tmpname = self.saveTemp(file, prefix=base + ".")
                return tmpname
            else:
                return file
        finally:
            if save and file:
                '''only close the file if we were asked to save it to a file'''
                file.close()

class HTTPImageFetcher(URIImageFetcher):
    
    def hasFile(self, filename):
        try:
            request = urllib2.Request(self.location + "/" + filename)
            request.get_method = lambda: "HEAD"
            http_file = urllib2.urlopen(request)
        except Exception, e:
            logging.debug("HTTP hasFile: didn't find %s" % \
                          (self.location + "/" + filename))
            return False
        return True

class FTPImageFetcher(URIImageFetcher):
    
    def hasFile(self, filename):
        url = urlparse.urlparse(self.location + "/" + filename)
        try:
            ftp = ftplib.FTP(url.hostname)
            if ftp.username or ftp.password:
                ftp.login(ftp.username, ftp.password)
            else:
                ftp.login()
            try:
                ftp.size(url[2])   # If a file
            except ftplib.all_errors, e:
                ftp.cwd(url[2])    # If a dir
        except ftplib.all_errors, e:
            logging.debug("FTP hasFile: couldn't access %s/%s" % \
                          (url[1], url[2]))
            return False
        return True

class LocalImageFetcher(ImageFetcher):

    def __init__(self, location, scratchdir, srcdir=None):
        ImageFetcher.__init__(self, location, scratchdir)
        self.srcdir = srcdir

    def _acquireFile(self, filename, progresscb, save=False):
        file = None
        try:
            logging.debug("Acquiring file from " + self.srcdir + "/" + filename)
            base = os.path.basename(filename)
            try:
                src = self.srcdir + "/" + filename
                if stat.S_ISDIR(os.stat(src)[stat.ST_MODE]):
                    logging.debug("Found a directory")
                    return None
                else:
                    # file = open(src, "r")
                    file = grabber.urlopen(self.srcdir + "/" + filename, \
                                           progress_obj = progresscb, \
                                          text = _("Retrieving file %s...") % base)
            except IOError, e:
                raise ValueError, _("Invalid file location given: ") + str(e)
            except OSError, (errno, msg):
                raise ValueError, _("Invalid file location given: ") + msg
            if save:
                tmpname = self.saveTemp(file, prefix=base + ".")
                return tmpname
            else:
                return file
        finally:
            if save and file:
                '''only close the file if we were asked to save it to a file'''
                file.close()

    def hasFile(self, filename):
        if os.path.exists(os.path.abspath(self.srcdir + "/" + filename)):
            return True
        else:
            logging.debug("local hasFile: Couldn't find %s" % \
                          (self.srcdir + "/" + filename))
            return False

# This is a fetcher capable of extracting files from a NFS server
# or loopback mounted file, or local CDROM device
class MountedImageFetcher(LocalImageFetcher):

    def prepareLocation(self, progresscb):

        self.need_umount = True
        self.srcdir = None

        # First, see if the nfs server:path is already mounted?
        (scheme,netloc,path) = self.location.split(':',2)
        for mount in snake.util.get_mtab(vfstype="nfs"):
            (mnt_hostname,mnt_path) = mount.mnt_fsname.split(":")
            if mnt_hostname == netloc and \
                path.startswith(mnt_path):
                self.srcdir = path.replace(mnt_path.rstrip('/'), mount.mnt_dir.rstrip('/'))

        # is the directory already mounted?
        if self.srcdir:
            if not os.path.exists(self.srcdir):
                raise ValueError(_("Location %s could not be found") % (self.location))
            # instruct the obj not to attempt a umount
            self.need_umount = False
            return os.path.exists(self.srcdir)

        # Otherwise, attempt to mount the nfs uri
        else:
            cmd = None
            self.srcdir = tempfile.mkdtemp(prefix="snakemnt.", dir=self.scratchdir)
            logging.debug("Preparing mount at " + self.srcdir)
            if self.location.startswith("nfs:"):
                cmd = ["mount", "-o", "ro", self.location[4:], self.srcdir]
            else:
                if stat.S_ISBLK(os.stat(self.location)[stat.ST_MODE]):
                    cmd = ["mount", "-o", "ro", self.location, self.srcdir]
                else:
                    cmd = ["mount", "-o", "ro,loop", self.location, self.srcdir]
            ret = snake.util.pcall(cmd, stderr=open('/dev/null', 'w'))
            if ret != 0:
                self.cleanupLocation()
                logging.debug("Mounting location %s failed" % (self.location,))
                raise ValueError(_("Mounting location %s failed") % (self.location))
                return False
            return True

    def cleanupLocation(self):
        if not self.need_umount:
            return
        logging.debug("Cleaning up mount at " + self.srcdir)
        cmd = ["umount", self.srcdir]
        ret = snake.util.pcall(cmd, stderr=open('/dev/null', 'w'))
        try:
            os.rmdir(self.srcdir)
        except:
            pass

class DirectImageFetcher(LocalImageFetcher):

    def prepareLocation(self, progresscb):
        self.srcdir = self.location

