#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#

import sys,socket,logging,dbus,avahi,gobject
import snake.dbushelper as dh
from dbus import glib
try:
    from dbus import DBusException          # new dbus API
except ImportError:
    from dbus_bindings import DBusException # dbus from 0.40.0 to 0.40.2

class Avahi(object):
    '''
    A generic base class for all SNAKE's avahi-related operations.'''
    def __init__(self):
        self.sbus = dbus.SystemBus()
        self.siface = dh.get_interface(self.sbus, avahi.DBUS_NAME, \
                         avahi.DBUS_PATH_SERVER, avahi.DBUS_INTERFACE_SERVER)

class AvahiServer(Avahi):
    '''
    AvahiServer class provides advertise() method to be used by
    on server side of SNAKE's architecture.'''
    def __init__(self):
        Avahi.__init__(self)
        self.log = logging.getLogger("snake")
        try:
            sgroup_entry = self.siface.EntryGroupNew()
            self.sgroup = dh.get_interface(self.sbus, avahi.DBUS_NAME,\
                          sgroup_entry, avahi.DBUS_INTERFACE_ENTRY_GROUP)
            assert self.sgroup.IsEmpty()
        except DBusException, message:
            self.log.error("Avahi error: %s" % message)

    def advertise(self, hostname, port,\
                  stype='_snake-server._tcp', sname="SNAKE",\
                  ai=avahi.IF_UNSPEC, ap=avahi.PROTO_UNSPEC,\
                  af=avahi.PUBLISH_UPDATE, ad='local'):
        '''
        Usage: AvahiServer.advertise(hostname, port, stype)
        Description: advertise new service using Avahi.
                     hostname: hostname/fqdn to bind the service to
                     port: port number to bind the service to'''
        sname = "%s [%s:%s]" % (sname, hostname, str(port)) # service name
        ah = socket.getfqdn(hostname)                   # host (fqdn)
        aport = dbus.UInt16(int(port))                  # port
        txt = avahi.string_array_to_txt_array([])       # service comment
        try:          # add specified service into created service group
            self.sgroup.AddService(ai, ap, af, sname, stype, ad, ah, aport, txt)
        except DBusException, message:
            self.log.error("Avahi error: %s" % message)
        else:
            self.log.info("Registered service %s of type %s at %s:%s" % \
                           (sname, stype, ah, aport))
            self.sgroup.Commit() # close the transaction
        return

class AvahiClient(Avahi):
    '''
    AvahiClient class provides get_server() method to be used
    on client side of SNAKE's architecture'''
    def __init__(self):
        Avahi.__init__(self)
        self.main_loop = gobject.MainLoop() # instantiate gobject event loop
        self.servers = []                   # list of servers found

    def find_servers(self, timeout=4, verbose=False, stype='_snake-server._tcp'):
        '''
        Usage: AvahiClient.get_server(timeout, verbose)
        Description: retrieve/resolve server information using Avahi.
                     timeout: time interval (in seconds) to resolve
                     verbose: print verbose information to stdout'''
        iface = avahi.IF_UNSPEC           # interface
        prot = avahi.PROTO_INET           # protocol
        dom = 'local'                     # domain
        flg = dbus.UInt32(0)              # flags
        # service browser object
        browser = self.siface.ServiceBrowserNew(iface, prot, stype, dom, flg)
        # service browser interface
        self.sbiface = dh.get_interface(self.sbus, avahi.DBUS_NAME, browser,\
                                        avahi.DBUS_INTERFACE_SERVICE_BROWSER)

        # new_item_callback will be called with every 'ItemNew' signal
        def new_item_callback(interface, protocol, name, type, domain, flags):
            '''Callback for "ItemNew" signal'''

            if verbose: print "Found service %s of type %s" % (name, type)
            try:
                server = self.siface.ResolveService(int(interface), int(protocol), \
                 name, type, domain, avahi.PROTO_UNSPEC, dbus.UInt32(0))
            except DBusException, message:
                print "Avahi error: %s" % message
            else:
                (riface, rprot, rname, rtype, rdomain, rhost, raproto, raddr, rport, rtxt, rflags) = server
                sinfo = (name, rhost, rport)
                if verbose:
                    print "Service resolved: %s:%s:%s" % sinfo

                if sinfo not in self.servers:
                    self.servers.append(sinfo)
                else:
                    if verbose:
                        print "Skipping resolved service: %s:%s:%s" % sinfo

        def all_for_now_callback():
            ''' This signal tells us that we now have all the service entires
                of the types we asked for, that avahi presently knows about.
                Exits the mainloop, returning control back to where the loop was
                first run.'''
            self.main_loop.quit() # <- quit the gobject event loop

        # register callback for 'ItemNew' signal
        self.sbiface.connect_to_signal('ItemNew', new_item_callback)
        self.sbiface.connect_to_signal('AllForNow', all_for_now_callback)
        try: # waiting loop
            def do_timeout(mainloop):
                mainloop.quit()
                if verbose: print "Timeout waiting for snake server."
                return False # only run once
            gobject.timeout_add(int(timeout*1000), do_timeout, self.main_loop)
            self.main_loop.run()
        except KeyboardInterrupt:
            pass
        return self.servers
